@@ -12,7 +12,10 @@
 #include "../../modules/cloudpinyin/cloudpinyin_public.h"
 #include "config.h"
 #include "punctuation.h"
+#include <cstdint>
 #include <ctime>
+#include <fcitx-utils/capabilityflags.h>
+#include <fcitx-utils/keysym.h>
 #include <fcitx-utils/keysymgen.h>
 #include <fcitx-utils/stringutils.h>
 #include <fcitx/candidatelist.h>
@@ -49,12 +52,12 @@
 #include <fmt/chrono.h>
 #include <fmt/format.h>
 #include <libime/core/historybigram.h>
-#include <libime/core/prediction.h>
 #include <libime/core/userlanguagemodel.h>
 #include <libime/pinyin/pinyincontext.h>
 #include <libime/pinyin/pinyindecoder.h>
 #include <libime/pinyin/pinyindictionary.h>
 #include <libime/pinyin/pinyinencoder.h>
+#include <libime/pinyin/pinyinprediction.h>
 #include <libime/pinyin/shuangpinprofile.h>
 #include <quickphrase_public.h>
 
@@ -128,6 +131,28 @@ public:
     std::string word_;
 };
 
+class PinyinPredictDictCandidateWord : public CandidateWord {
+public:
+    PinyinPredictDictCandidateWord(PinyinEngine *engine, std::string word)
+        : CandidateWord(Text(word)), engine_(engine), word_(std::move(word)) {}
+
+    void select(InputContext *inputContext) const override {
+        inputContext->commitString(word_);
+        auto *state = inputContext->propertyFor(&engine_->factory());
+        if (!state->predictWords_) {
+            state->predictWords_.emplace();
+        }
+        // Append to last word, instead of push back.
+        if (!state->predictWords_->empty()) {
+            state->predictWords_->back().append(word_);
+        }
+        engine_->updatePredict(inputContext);
+    }
+
+    PinyinEngine *engine_;
+    std::string word_;
+};
+
 class PinyinAbstractExtraCandidateWordInterface {
 public:
     explicit PinyinAbstractExtraCandidateWordInterface(CandidateWord &cand,
@@ -339,15 +364,41 @@ class CustomCloudPinyinCandidateWord
     : public CloudPinyinCandidateWord,
       public PinyinAbstractExtraCandidateWordInterface {
 public:
-    CustomCloudPinyinCandidateWord(AddonInstance *cloudpinyin,
+    CustomCloudPinyinCandidateWord(PinyinEngine *engine,
                                    const std::string &pinyin,
                                    const std::string &selectedSentence,
                                    InputContext *inputContext,
                                    CloudPinyinSelectedCallback callback,
                                    int order)
-        : CloudPinyinCandidateWord(cloudpinyin, pinyin, selectedSentence,
+        : CloudPinyinCandidateWord(engine->cloudpinyin(), pinyin,
+                                   selectedSentence,
+                                   *engine->config().keepCloudPinyinPlaceHolder,
                                    inputContext, callback),
-          PinyinAbstractExtraCandidateWordInterface(*this, order) {}
+          PinyinAbstractExtraCandidateWordInterface(*this, order) {
+        if (filled() || !*engine->config().cloudPinyinAnimation) {
+            return;
+        }
+        setText(Text(std::string(ProgerssString[tick_])));
+        // This should be high accuracy since it's per 120ms.
+        timeEvent_ = engine->instance()->eventLoop().addTimeEvent(
+            CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + TickPeriod, 1000,
+            [this, ref = this->watch()](EventSourceTime *, uint64_t time) {
+                if (!ref.isValid()) {
+                    return true;
+                }
+                if (filled()) {
+                    timeEvent_.reset();
+                    return true;
+                }
+                tick_ = (time / TickPeriod) % ProgerssString.size();
+                setText(Text(std::string(ProgerssString[tick_])));
+                this->inputContext()->updateUserInterface(
+                    fcitx::UserInterfaceComponent::InputPanel);
+                timeEvent_->setTime(timeEvent_->time() + TickPeriod);
+                timeEvent_->setOneShot();
+                return true;
+            });
+    }
 
     void select(InputContext *inputContext) const override {
         if ((!filled() || word().empty()) && order() == 0) {
@@ -360,19 +411,46 @@ public:
         }
         CloudPinyinCandidateWord::select(inputContext);
     }
+
+private:
+    static constexpr std::array<std::string_view, 4> ProgerssString = {
+        "◐",
+        "◓",
+        "◑",
+        "◒",
+    };
+    int tick_ = (now(CLOCK_MONOTONIC) / TickPeriod) % ProgerssString.size();
+    std::unique_ptr<EventSourceTime> timeEvent_;
+    static constexpr uint64_t TickPeriod = 180000;
 };
 
+template <typename T>
 std::unique_ptr<CandidateList>
-PinyinEngine::predictCandidateList(const std::vector<std::string> &words) {
+predictCandidateList(PinyinEngine *engine, const std::vector<T> &words) {
     if (words.empty()) {
         return nullptr;
     }
     auto candidateList = std::make_unique<CommonCandidateList>();
     for (const auto &word : words) {
-        candidateList->append<PinyinPredictCandidateWord>(this, word);
+        if constexpr (std::is_same_v<T, std::string>) {
+            candidateList->append<PinyinPredictCandidateWord>(engine, word);
+        } else if constexpr (std::is_same_v<
+                                 T,
+                                 std::pair<std::string,
+                                           libime::PinyinPredictionSource>>) {
+
+            if (word.second == libime::PinyinPredictionSource::Model) {
+                candidateList->append<PinyinPredictCandidateWord>(engine,
+                                                                  word.first);
+            } else if (word.second ==
+                       libime::PinyinPredictionSource::Dictionary) {
+                candidateList->append<PinyinPredictDictCandidateWord>(
+                    engine, word.first);
+            }
+        }
     }
-    candidateList->setSelectionKey(selectionKeys_);
-    candidateList->setPageSize(*config_.pageSize);
+    candidateList->setSelectionKey(engine->selectionKeys());
+    candidateList->setPageSize(*engine->config().pageSize);
     if (candidateList->size()) {
         candidateList->setGlobalCursorIndex(0);
     }
@@ -386,9 +464,11 @@ void PinyinEngine::initPredict(InputContext *inputContext) {
     auto &context = state->context_;
     auto lmState = context.state();
     state->predictWords_ = context.selectedWords();
-    auto words = prediction_.predict(lmState, context.selectedWords(),
-                                     *config_.predictionSize);
-    if (auto candidateList = predictCandidateList(words)) {
+    auto words =
+        prediction_.predict(lmState, context.selectedWords(),
+                            context.selectedWordsWithPinyin().back().second,
+                            *config_.predictionSize);
+    if (auto candidateList = predictCandidateList(this, words)) {
         auto &inputPanel = inputContext->inputPanel();
         inputPanel.setCandidateList(std::move(candidateList));
     } else {
@@ -405,7 +485,7 @@ void PinyinEngine::updatePredict(InputContext *inputContext) {
     assert(state->predictWords_.has_value());
     auto words =
         prediction_.predict(*state->predictWords_, *config_.predictionSize);
-    if (auto candidateList = predictCandidateList(words)) {
+    if (auto candidateList = predictCandidateList(this, words)) {
         auto &inputPanel = inputContext->inputPanel();
         inputPanel.setCandidateList(std::move(candidateList));
     } else {
@@ -488,11 +568,17 @@ PinyinEngine::luaCandidateTrigger(InputContext *ic,
 #endif
 std::pair<Text, Text> PinyinEngine::preedit(InputContext *inputContext) const {
     auto *state = inputContext->propertyFor(&factory_);
+    PreeditMode mode =
+        inputContext->capabilityFlags().test(CapabilityFlag::Preedit)
+            ? *config_.preeditMode
+            : PreeditMode::No;
     // Use const ref to avoid accidentally change anything.
     const auto &context = state->context_;
     auto preeditWithCursor = context.preeditWithCursor();
-    Text clientPreedit;
-    if (*config_.showPreeditInApplication) {
+    // client preedit can be empty/pinyin/preview depends on config
+    Text clientPreedit, preedit;
+    switch (mode) {
+    case PreeditMode::ComposingPinyin:
         if (*config_.preeditCursorPositionAtBeginning) {
             clientPreedit.append(
                 preeditWithCursor.first.substr(0, preeditWithCursor.second),
@@ -506,17 +592,20 @@ std::pair<Text, Text> PinyinEngine::preedit(InputContext *inputContext) const {
                                  TextFormatFlag::Underline);
             clientPreedit.setCursor(preeditWithCursor.second);
         }
-    } else {
+        break;
+    case PreeditMode::CommitPreview:
         clientPreedit.append(context.sentence(), TextFormatFlag::Underline);
         if (*config_.preeditCursorPositionAtBeginning) {
             clientPreedit.setCursor(0);
         } else {
             clientPreedit.setCursor(context.selectedSentence().size());
         }
+        [[fallthrough]];
+    case PreeditMode::No:
+        preedit.append(preeditWithCursor.first);
+        preedit.setCursor(preeditWithCursor.second);
+        break;
     }
-
-    Text preedit(preeditWithCursor.first);
-    preedit.setCursor(preeditWithCursor.second);
     return {std::move(clientPreedit), std::move(preedit)};
 }
 
@@ -537,14 +626,8 @@ PinyinEngine::preeditCommitString(InputContext *inputContext) const {
 void PinyinEngine::updatePreedit(InputContext *inputContext) const {
     auto &inputPanel = inputContext->inputPanel();
     auto [clientPreedit, preedit] = this->preedit(inputContext);
-    if (inputContext->capabilityFlags().test(CapabilityFlag::Preedit)) {
-        inputPanel.setClientPreedit(clientPreedit);
-    }
-
-    if (!config_.showPreeditInApplication.value() ||
-        !inputContext->capabilityFlags().test(CapabilityFlag::Preedit)) {
-        inputPanel.setPreedit(preedit);
-    }
+    inputPanel.setClientPreedit(clientPreedit);
+    inputPanel.setPreedit(preedit);
 }
 
 void PinyinEngine::updatePuncPreedit(InputContext *inputContext) const {
@@ -682,7 +765,7 @@ void PinyinEngine::updateUI(InputContext *inputContext) {
                                   ? context.candidateFullPinyin(0)
                                   : context.userInput().substr(selectedLength);
             auto cand = std::make_unique<CustomCloudPinyinCandidateWord>(
-                cloudpinyin(), fullPinyin, selectedSentence, inputContext,
+                this, fullPinyin, selectedSentence, inputContext,
                 std::bind(&PinyinEngine::cloudPinyinSelected, this, _1, _2, _3),
                 *config_.cloudPinyinIndex - 1);
             if (!cand->filled() ||
@@ -826,6 +909,8 @@ std::string PinyinEngine::evaluateCustomPhrase(InputContext *inputContext,
     if (!result.empty()) {
         return result;
     }
+
+#ifdef FCITX_HAS_LUA
     if (stringutils::startsWith(key, "lua:")) {
         RawConfig config;
         auto ret = imeapi()->call<ILuaAddon::invokeLuaFunction>(
@@ -834,6 +919,7 @@ std::string PinyinEngine::evaluateCustomPhrase(InputContext *inputContext,
             return ret.value();
         }
     }
+#endif
     return "";
 }
 
@@ -863,6 +949,7 @@ PinyinEngine::PinyinEngine(Instance *instance)
                            libime::PinyinDictFormat::Binary);
     }
     prediction_.setUserLanguageModel(ime_->model());
+    prediction_.setPinyinDictionary(ime_->dict());
 
     do {
         auto file = standardPath.openUser(StandardPath::Type::PkgData,
@@ -963,19 +1050,22 @@ PinyinEngine::PinyinEngine(Instance *instance)
     pyConfig_.showShuangpinMode.annotation().setHidden(true);
     pyConfig_.fuzzyConfig->partialSp.annotation().setHidden(true);
 
-    checkCloudPinyinAvailable_ =
-        instance_->eventLoop().addDeferEvent([this](EventSource *) {
-            bool hasCloudPinyin = cloudpinyin() != nullptr;
-            for (auto *configPtr : {&config_, &pyConfig_}) {
-                configPtr->cloudPinyinEnabled.annotation().setHidden(
-                    !hasCloudPinyin);
-                configPtr->cloudPinyinIndex.annotation().setHidden(
-                    !hasCloudPinyin);
-                configPtr->cloudpinyin.setHidden(!hasCloudPinyin);
-            }
-            checkCloudPinyinAvailable_.reset();
-            return true;
-        });
+    deferredPreload_ = instance_->eventLoop().addDeferEvent([this](
+                                                                EventSource *) {
+        bool hasCloudPinyin = cloudpinyin() != nullptr;
+        for (auto *configPtr : {&config_, &pyConfig_}) {
+            configPtr->cloudPinyinEnabled.annotation().setHidden(
+                !hasCloudPinyin);
+            configPtr->cloudPinyinIndex.annotation().setHidden(!hasCloudPinyin);
+            configPtr->cloudPinyinAnimation.annotation().setHidden(
+                !hasCloudPinyin);
+            configPtr->keepCloudPinyinPlaceHolder.annotation().setHidden(
+                !hasCloudPinyin);
+            configPtr->cloudpinyin.setHidden(!hasCloudPinyin);
+        }
+        deferredPreload_.reset();
+        return true;
+    });
 }
 
 PinyinEngine::~PinyinEngine() {}
@@ -1172,6 +1262,7 @@ void PinyinEngine::populateConfig() {
     }
     SET_FUZZY_FLAG(ue, VE_UE)
     SET_FUZZY_FLAG(commonTypo, CommonTypo)
+    SET_FUZZY_FLAG(commonTypo, AdvancedTypo)
     SET_FUZZY_FLAG(inner, Inner)
     SET_FUZZY_FLAG(innerShort, InnerShort)
     SET_FUZZY_FLAG(partialFinal, PartialFinal)
@@ -1242,9 +1333,17 @@ void PinyinEngine::reloadConfig() {
 void PinyinEngine::activate(const fcitx::InputMethodEntry &entry,
                             fcitx::InputContextEvent &event) {
     auto *inputContext = event.inputContext();
-    // Request full width.
+    // Request addons that gonna be used but not listed in optional
+    // dependencies.
     fullwidth();
     chttrans();
+    if (*config_.spellEnabled) {
+        spell();
+    }
+    if (pinyinhelper()) {
+        // Preload stroke data, since we gonna use it anyway.
+        pinyinhelper()->call<IPinyinHelper::loadStroke>();
+    }
     for (const auto *actionName : {"chttrans", "punctuation", "fullwidth"}) {
         if (auto *action =
                 instance_->userInterfaceManager().lookupAction(actionName)) {
@@ -1266,6 +1365,18 @@ void PinyinEngine::deactivate(const fcitx::InputMethodEntry &entry,
             break;
         }
         auto *state = inputContext->propertyFor(&factory_);
+        if (state->mode_ == PinyinMode::Punctuation) {
+            auto candidateList = inputContext->inputPanel().candidateList();
+            if (!candidateList) {
+                break;
+            }
+            auto index = candidateList->cursorIndex();
+            if (index >= 0) {
+                candidateList->candidate(index).select(inputContext);
+            }
+            break;
+        }
+
         if (state->context_.empty()) {
             break;
         }
@@ -1301,34 +1412,36 @@ bool PinyinEngine::handlePuncCandidate(KeyEvent &event) {
         doReset(inputContext);
         return true;
     }
-    int idx = event.key().keyListIndex(selectionKeys_);
-    if (idx == -1 && *config_.useKeypadAsSelectionKey) {
-        idx = event.key().keyListIndex(numpadSelectionKeys_);
-    }
-    if (idx >= 0) {
-        event.filterAndAccept();
-        if (idx < candidateList->size()) {
-            candidateList->candidate(idx).select(inputContext);
+    if (!event.isVirtual()) {
+        int idx = event.key().keyListIndex(selectionKeys_);
+        if (idx == -1 && *config_.useKeypadAsSelectionKey) {
+            idx = event.key().keyListIndex(numpadSelectionKeys_);
+        }
+        if (idx >= 0) {
+            event.filterAndAccept();
+            if (idx < candidateList->size()) {
+                candidateList->candidate(idx).select(inputContext);
+            }
+            return true;
         }
-        return true;
-    }
 
-    if (auto *movable = candidateList->toCursorMovable()) {
-        if (event.key().checkKeyList(*config_.nextCandidate)) {
-            movable->nextCandidate();
-            updatePuncPreedit(inputContext);
-            inputContext->updateUserInterface(
-                UserInterfaceComponent::InputPanel);
-            event.filterAndAccept();
-            return true;
-        }
-        if (event.key().checkKeyList(*config_.prevCandidate)) {
-            movable->prevCandidate();
-            updatePuncPreedit(inputContext);
-            inputContext->updateUserInterface(
-                UserInterfaceComponent::InputPanel);
-            event.filterAndAccept();
-            return true;
+        if (auto *movable = candidateList->toCursorMovable()) {
+            if (event.key().checkKeyList(*config_.nextCandidate)) {
+                movable->nextCandidate();
+                updatePuncPreedit(inputContext);
+                inputContext->updateUserInterface(
+                    UserInterfaceComponent::InputPanel);
+                event.filterAndAccept();
+                return true;
+            }
+            if (event.key().checkKeyList(*config_.prevCandidate)) {
+                movable->prevCandidate();
+                updatePuncPreedit(inputContext);
+                inputContext->updateUserInterface(
+                    UserInterfaceComponent::InputPanel);
+                event.filterAndAccept();
+                return true;
+            }
         }
     }
 
@@ -1353,7 +1466,7 @@ bool PinyinEngine::handleCloudpinyinTrigger(KeyEvent &event) {
                 _("Cloud Pinyin Status"),
                 *config_.cloudPinyinEnabled ? _("Cloud Pinyin is enabled.")
                                             : _("Cloud Pinyin is disabled."),
-                -1);
+                1000);
         }
         if (*config_.cloudPinyinEnabled) {
             cloudpinyin()->call<ICloudPinyin::resetError>();
@@ -1439,17 +1552,6 @@ bool PinyinEngine::handleCandidateList(KeyEvent &event) {
     if (!candidateList) {
         return false;
     }
-    int idx = event.key().keyListIndex(selectionKeys_);
-    if (idx == -1 && *config_.useKeypadAsSelectionKey) {
-        idx = event.key().keyListIndex(numpadSelectionKeys_);
-    }
-    if (idx >= 0) {
-        event.filterAndAccept();
-        if (idx < candidateList->size()) {
-            candidateList->candidate(idx).select(inputContext);
-        }
-        return true;
-    }
     auto *state = inputContext->propertyFor(&factory_);
     if ((event.key().check(FcitxKey_space) ||
          event.key().check(FcitxKey_KP_Space)) &&
@@ -1465,6 +1567,22 @@ bool PinyinEngine::handleCandidateList(KeyEvent &event) {
         }
     }
 
+    if (event.isVirtual()) {
+        return false;
+    }
+
+    int idx = event.key().keyListIndex(selectionKeys_);
+    if (idx == -1 && *config_.useKeypadAsSelectionKey) {
+        idx = event.key().keyListIndex(numpadSelectionKeys_);
+    }
+    if (idx >= 0) {
+        event.filterAndAccept();
+        if (idx < candidateList->size()) {
+            candidateList->candidate(idx).select(inputContext);
+        }
+        return true;
+    }
+
     if (event.key().checkKeyList(*config_.prevPage)) {
         auto *pageable = candidateList->toPageable();
         if (!pageable->hasPrev()) {
@@ -1785,17 +1903,18 @@ bool PinyinEngine::handlePunc(KeyEvent &event) {
         auto candidates =
             punctuation()->call<IPunctuation::getPunctuationCandidates>("zh_CN",
                                                                         c);
+        auto pushResult = punctuation()->call<IPunctuation::pushPunctuationV2>(
+            "zh_CN", inputContext, c);
         if (candidates.size() == 1) {
-            std::tie(punc, puncAfter) =
-                punctuation()->call<IPunctuation::pushPunctuationV2>(
-                    "zh_CN", inputContext, c);
+            std::tie(punc, puncAfter) = pushResult;
         } else if (candidates.size() > 1) {
             updatePuncCandidate(inputContext, utf8::UCS4ToUTF8(c), candidates);
             event.filterAndAccept();
             return true;
         }
     }
-    if (event.key().check(*config_.quickphraseKey) && quickphrase()) {
+    if (!event.isVirtual() && event.key().check(*config_.quickphraseKey) &&
+        quickphrase()) {
         auto keyString = utf8::UCS4ToUTF8(c);
         // s is punc or key
         auto output = !punc.empty() ? (punc + puncAfter) : keyString;
@@ -1820,11 +1939,22 @@ bool PinyinEngine::handlePunc(KeyEvent &event) {
     }
     if (!punc.empty()) {
         event.filterAndAccept();
-        inputContext->commitString(punc + puncAfter);
-        if (size_t length = utf8::lengthValidated(puncAfter);
-            length != 0 && length != utf8::INVALID_LENGTH) {
-            for (size_t i = 0; i < length; i++) {
-                inputContext->forwardKey(Key(FcitxKey_Left));
+        auto paired = punc + puncAfter;
+        if (inputContext->capabilityFlags().test(
+                CapabilityFlag::CommitStringWithCursor)) {
+            if (size_t length = utf8::lengthValidated(punc);
+                length != 0 && length != utf8::INVALID_LENGTH) {
+                inputContext->commitStringWithCursor(paired, length);
+            } else {
+                inputContext->commitString(paired);
+            }
+        } else {
+            inputContext->commitString(paired);
+            if (size_t length = utf8::lengthValidated(puncAfter);
+                length != 0 && length != utf8::INVALID_LENGTH) {
+                for (size_t i = 0; i < length; i++) {
+                    inputContext->forwardKey(Key(FcitxKey_Left));
+                }
             }
         }
     }
@@ -1832,6 +1962,50 @@ bool PinyinEngine::handlePunc(KeyEvent &event) {
     return false;
 }
 
+bool PinyinEngine::handleCompose(KeyEvent &event) {
+    auto *inputContext = event.inputContext();
+    auto *state = inputContext->propertyFor(&factory_);
+    if (event.key().hasModifier() || state->mode_ != PinyinMode::Normal) {
+        return false;
+    }
+    auto candidateList = inputContext->inputPanel().candidateList();
+    if (event.filtered()) {
+        return false;
+    }
+    auto compose =
+        instance_->processComposeString(inputContext, event.key().sym());
+    if (!compose) {
+        // invalid compose or in the middle of compose.
+        event.filterAndAccept();
+        return true;
+    }
+    // Handle the key just like punc select.
+    if (!compose->empty()) {
+        // Reset predict in case we are in predict.
+        resetPredict(inputContext);
+        // punc like auto selection.
+        auto candidateList = inputContext->inputPanel().candidateList();
+        if (candidateList && candidateList->size()) {
+            candidateList->candidate(0).select(inputContext);
+        }
+        inputContext->commitString(*compose);
+        event.filterAndAccept();
+        return true;
+    }
+    return false;
+}
+
+void PinyinEngine::resetPredict(InputContext *inputContext) {
+    auto *state = inputContext->propertyFor(&factory_);
+    if (!state->predictWords_) {
+        return;
+    }
+    state->predictWords_.reset();
+    inputContext->inputPanel().reset();
+    inputContext->updatePreedit();
+    inputContext->updateUserInterface(UserInterfaceComponent::InputPanel);
+}
+
 void PinyinEngine::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
     FCITX_UNUSED(entry);
     PINYIN_DEBUG() << "Pinyin receive key: " << event.key() << " "
@@ -1871,6 +2045,10 @@ void PinyinEngine::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
         return;
     }
 
+    if (handleCompose(event)) {
+        return;
+    }
+
     // handle number key selection and prev/next page/candidate.
     if (handleCandidateList(event)) {
         return;
@@ -1884,16 +2062,10 @@ void PinyinEngine::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
     // fallback
     // to remaining operation.
     if (state->predictWords_) {
-        state->predictWords_.reset();
-        inputContext->inputPanel().reset();
-        inputContext->updatePreedit();
-        inputContext->updateUserInterface(UserInterfaceComponent::InputPanel);
-        if (event.key().check(FcitxKey_Escape)
-#ifdef ANDROID
-            || event.key().check(FcitxKey_BackSpace) ||
-            event.key().check(FcitxKey_Delete)
-#endif
-        ) {
+        resetPredict(inputContext);
+        if (event.key().check(FcitxKey_Escape) ||
+            (isAndroid() && event.key().check(FcitxKey_BackSpace)) ||
+            event.key().check(FcitxKey_Delete)) {
             event.filterAndAccept();
             return;
         }
@@ -2059,7 +2231,6 @@ void PinyinEngine::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
                 auto puncStr = punctuation()->call<IPunctuation::cancelLast>(
                     "zh_CN", inputContext);
                 if (!puncStr.empty()) {
-                    // forward the original key is the best choice.
                     // forward the original key is the best choice.
                     auto ref = inputContext->watch();
                     state->cancelLastEvent_ =
@@ -2121,6 +2292,7 @@ void PinyinEngine::doReset(InputContext *inputContext) const {
 
     state->keyReleased_ = -1;
     state->keyReleasedIndex_ = -2;
+    instance_->resetCompose(inputContext);
 }
 
 void PinyinEngine::save() {
@@ -2207,7 +2379,10 @@ void PinyinEngine::invokeActionImpl(const InputMethodEntry &entry,
     auto preeditWithCursor = context.preeditWithCursor();
     auto selectedSentence = context.selectedSentence();
     // The logic here need to match ::preedit()
-    if (*config_.showPreeditInApplication) {
+    switch (*config_.preeditMode) {
+    case PreeditMode::No:
+        break;
+    case PreeditMode::ComposingPinyin:
         if (utf8::length(selectedSentence) > cursor) {
             // If cursor is with in selected sentence range, cancel until cursor
             // is covered.
@@ -2237,12 +2412,14 @@ void PinyinEngine::invokeActionImpl(const InputMethodEntry &entry,
                 state->context_.setCursor(context.cursor() - 1);
             }
         }
-    } else {
+        break;
+    case PreeditMode::CommitPreview:
         if (utf8::length(selectedSentence) > cursor) {
             do {
                 context.cancel();
             } while (utf8::length(context.selectedSentence()) > cursor);
         }
+        break;
     }
     updateUI(inputContext);
 }
