@@ -12,6 +12,7 @@
 #include <fcitx-config/iniparser.h>
 #include <fcitx-utils/event.h>
 #include <fcitx-utils/i18n.h>
+#include <fcitx-utils/misc.h>
 #include <fcitx-utils/standardpath.h>
 #include <fcitx/action.h>
 #include <fcitx/addonfactory.h>
@@ -21,9 +22,9 @@
 #include <fcitx/inputcontextproperty.h>
 #include <fcitx/inputmethodengine.h>
 #include <fcitx/instance.h>
-#include <libime/core/prediction.h>
 #include <libime/pinyin/pinyincontext.h>
 #include <libime/pinyin/pinyinime.h>
+#include <libime/pinyin/pinyinprediction.h>
 #include <memory>
 #include <string_view>
 #include <unordered_map>
@@ -31,12 +32,6 @@
 
 namespace fcitx {
 
-#ifdef ANDROID
-static constexpr bool isAndroid = true;
-#else
-static constexpr bool isAndroid = false;
-#endif
-
 template <typename Base = NoAnnotation>
 struct OptionalHideInDescriptionBase : public Base {
     void setHidden(bool hidden) { hidden_ = hidden; }
@@ -77,6 +72,11 @@ FCITX_CONFIG_ENUM_I18N_ANNOTATION(ShuangpinProfileEnum, N_("Ziranma"), N_("MS"),
                                   N_("Zhongwenzhixing"), N_("PinyinJiajia"),
                                   N_("Xiaohe"), N_("Custom"))
 
+enum class PreeditMode { No, ComposingPinyin, CommitPreview };
+
+FCITX_CONFIG_ENUM_NAME_WITH_I18N(PreeditMode, N_("Do not show"),
+                                 N_("Composing pinyin"), N_("Commit preview"))
+
 FCITX_CONFIGURATION(
     FuzzyConfig, Option<bool> ue{this, "VE_UE", _("ue -> ve"), true};
     Option<bool> commonTypo{this, "NG_GN", _("Common Typo"), true};
@@ -119,25 +119,35 @@ FCITX_CONFIGURATION(
     Option<bool> chaiziEnabled{this, "ChaiziEnabled", _("Enable Chaizi"), true};
     Option<bool> extBEnabled{this, "ExtBEnabled",
                              _("Enable Characters in Unicode CJK Extension B"),
-                             !isAndroid};
+                             !isAndroid()};
+    OptionalHiddenSubConfigOption cloudpinyin{
+        this, "CloudPinyin", _("Cloud Pinyin"),
+        "fcitx://config/addon/cloudpinyin"};
     OptionWithAnnotation<bool, OptionalHideInDescription> cloudPinyinEnabled{
         this, "CloudPinyinEnabled", _("Enable Cloud Pinyin"), false};
     Option<int, IntConstrain, DefaultMarshaller<int>, OptionalHideInDescription>
         cloudPinyinIndex{this, "CloudPinyinIndex", _("Cloud Pinyin Index"), 2,
                          IntConstrain(1, 10)};
-    Option<bool> showPreeditInApplication{this, "PreeditInApplication",
-                                          _("Show preedit within application"),
-                                          true};
+    OptionWithAnnotation<bool, OptionalHideInDescription> cloudPinyinAnimation{
+        this, "CloudPinyinAnimation",
+        _("Show animation when Cloud Pinyin is loading"), true};
+    OptionWithAnnotation<bool, OptionalHideInDescription>
+        keepCloudPinyinPlaceHolder{this, "KeepCloudPinyinPlaceHolder",
+                                   _("Always show Cloud Pinyin place holder"),
+                                   false};
+    OptionWithAnnotation<PreeditMode, PreeditModeI18NAnnotation> preeditMode{
+        this, "PreeditMode", _("Preedit Mode"),
+        isAndroid() ? PreeditMode::No : PreeditMode::ComposingPinyin};
     Option<bool> preeditCursorPositionAtBeginning{
         this, "PreeditCursorPositionAtBeginning",
         _("Fix embedded preedit cursor at the beginning of the preedit"),
-        !isAndroid};
+        !isAndroid()};
     Option<bool> showActualPinyinInPreedit{
         this, "PinyinInPreedit", _("Show complete pinyin in preedit"), false};
     Option<bool> predictionEnabled{this, "Prediction", _("Enable Prediction"),
-                                   false};
+                                   isAndroid()};
     Option<int, IntConstrain> predictionSize{
-        this, "PredictionSize", _("Prediction Size"), 10, IntConstrain(3, 20)};
+        this, "PredictionSize", _("Prediction Size"), 49, IntConstrain(3, 100)};
     OptionWithAnnotation<SwitchInputMethodBehavior,
                          SwitchInputMethodBehaviorI18NAnnotation>
         switchInputMethodBehavior{this, "SwitchInputMethodBehavior",
@@ -220,9 +230,6 @@ FCITX_CONFIGURATION(
     SubConfigOption chttrans{
         this, "Chttrans", _("Simplified and Traditional Chinese Translation"),
         "fcitx://config/addon/chttrans"};
-    OptionalHiddenSubConfigOption cloudpinyin{
-        this, "CloudPinyin", _("Cloud Pinyin"),
-        "fcitx://config/addon/cloudpinyin"};
     Option<Key, KeyConstrain> quickphraseKey{
         this,
         "QuickPhraseKey",
@@ -285,16 +292,20 @@ public:
                       const fcitx::RawConfig &) override;
 
     libime::PinyinIME *ime() { return ime_.get(); }
+    const auto &config() const { return config_; }
 
     void initPredict(InputContext *inputContext);
     void updatePredict(InputContext *inputContext);
-    std::unique_ptr<CandidateList>
-    predictCandidateList(const std::vector<std::string> &words);
+
     void updateUI(InputContext *inputContext);
 
     void resetStroke(InputContext *inputContext) const;
     void resetForgetCandidate(InputContext *inputContext) const;
 
+    FCITX_ADDON_DEPENDENCY_LOADER(cloudpinyin, instance_->addonManager());
+
+    const auto &selectionKeys() const { return selectionKeys_; }
+
 private:
     void cloudPinyinSelected(InputContext *inputContext,
                              const std::string &selected,
@@ -308,6 +319,8 @@ private:
     bool handleForgetCandidate(KeyEvent &event);
     bool handlePunc(KeyEvent &event);
     bool handlePuncCandidate(KeyEvent &event);
+    bool handleCompose(KeyEvent &event);
+    void resetPredict(InputContext *inputContext);
 
     std::string evaluateCustomPhrase(InputContext *inputContext,
                                      std::string_view key);
@@ -345,14 +358,13 @@ private:
     KeyList numpadSelectionKeys_;
     FactoryFor<PinyinState> factory_;
     SimpleAction predictionAction_;
-    libime::Prediction prediction_;
+    libime::PinyinPrediction prediction_;
     std::unique_ptr<EventSource> deferEvent_;
-    std::unique_ptr<EventSource> checkCloudPinyinAvailable_;
+    std::unique_ptr<EventSource> deferredPreload_;
     std::unique_ptr<HandlerTableEntry<EventHandler>> event_;
     CustomPhraseDict customPhrase_;
 
     FCITX_ADDON_DEPENDENCY_LOADER(quickphrase, instance_->addonManager());
-    FCITX_ADDON_DEPENDENCY_LOADER(cloudpinyin, instance_->addonManager());
     FCITX_ADDON_DEPENDENCY_LOADER(fullwidth, instance_->addonManager());
     FCITX_ADDON_DEPENDENCY_LOADER(chttrans, instance_->addonManager());
     FCITX_ADDON_DEPENDENCY_LOADER(punctuation, instance_->addonManager());
