@@ -50,11 +50,7 @@ TableEngine::TableEngine(Instance *instance)
     events_.emplace_back(instance_->watchEvent(
         EventType::InputMethodGroupChanged, EventWatcherPhase::Default,
         [this](Event &) {
-            instance_->inputContextManager().foreach([this](InputContext *ic) {
-                auto *state = ic->propertyFor(&factory_);
-                state->release();
-                return true;
-            });
+            releaseStates();
             std::unordered_set<std::string> names;
             for (const auto &im : instance_->inputMethodManager()
                                       .currentGroup()
@@ -164,6 +160,12 @@ void TableEngine::populateConfig() {
     }
 }
 
+void TableEngine::setSubConfig(const std::string &path, const RawConfig &) {
+    if (path == "reloaddict") {
+        reloadDict();
+    }
+}
+
 void TableEngine::activate(const fcitx::InputMethodEntry &entry,
                            fcitx::InputContextEvent &event) {
     auto *inputContext = event.inputContext();
@@ -225,9 +227,20 @@ void TableEngine::reset(const InputMethodEntry &entry,
     auto *inputContext = event.inputContext();
 
     auto *state = inputContext->propertyFor(&factory_);
-    // The reason that we do not commit here is we want to force the behavior.
-    // When client get unfocused, the framework will try to commit the string.
-    if (state->context() && *state->context()->config().commitWhenDeactivate) {
+
+    if (state->mode() == TableMode::Punctuation) {
+        auto candidateList = inputContext->inputPanel().candidateList();
+        if (candidateList && event.type() != EventType::InputContextFocusOut) {
+            auto index = candidateList->cursorIndex();
+            if (index >= 0) {
+                candidateList->candidate(index).select(inputContext);
+            }
+        }
+    } else if (state->context() &&
+               *state->context()->config().commitWhenDeactivate) {
+        // The reason that we do not commit here is we want to force the
+        // behavior. When client get unfocused, the framework will try to commit
+        // the string.
         state->commitBuffer(true,
                             event.type() == EventType::InputContextFocusOut);
     }
@@ -289,6 +302,20 @@ void TableEngine::setConfigForInputMethod(const InputMethodEntry &entry,
                                           const RawConfig &config) {
     ime_->updateConfig(entry.uniqueName(), config);
 }
+
+void TableEngine::releaseStates() {
+    instance_->inputContextManager().foreach([&](InputContext *ic) {
+        auto *state = ic->propertyFor(&factory_);
+        state->release();
+        return true;
+    });
+}
+
+void TableEngine::reloadDict() {
+    releaseStates();
+    ime_->reloadAllDict();
+}
+
 } // namespace fcitx
 
 FCITX_ADDON_FACTORY(fcitx::TableEngineFactory)
