@@ -10,8 +10,10 @@
 #include "quickphrase_public.h"
 #include <algorithm>
 #include <fcitx-utils/event.h>
+#include <fcitx-utils/keysym.h>
 #include <fcitx-utils/stringutils.h>
 #include <fcitx-utils/utf8.h>
+#include <fcitx/event.h>
 #include <fcitx/inputcontext.h>
 #include <fcitx/inputpanel.h>
 #include <fmt/core.h>
@@ -254,6 +256,8 @@ void TableState::reset(const InputMethodEntry *entry) {
 
     keyReleased_ = -1;
     keyReleasedIndex_ = -2;
+    // Since we also to compose, just reset compose all together
+    engine_->instance()->resetCompose(ic_);
 }
 
 class TablePredictCandidateWord : public CandidateWord {
@@ -353,8 +357,11 @@ bool TableState::autoSelectCandidate() {
     return false;
 }
 
-bool TableState::handleCandidateList(const TableConfig &config, KeyEvent &event,
-                                     bool &needUpdate) {
+bool TableState::handleCandidateList(const TableConfig &config,
+                                     KeyEvent &event) {
+    if (event.isVirtual()) {
+        return false;
+    }
     auto *inputContext = event.inputContext();
     // check if we can select candidate.
     auto candidateList = inputContext->inputPanel().candidateList();
@@ -416,13 +423,6 @@ bool TableState::handleCandidateList(const TableConfig &config, KeyEvent &event,
         }
     }
 
-    // Non candidate key for predict candidate should clear it.
-    if (dynamic_cast<const TablePredictCandidateWord *>(
-            &candidateList->candidate(0))) {
-        inputContext->inputPanel().setCandidateList(nullptr);
-        needUpdate = true;
-    }
-
     return false;
 }
 
@@ -466,7 +466,7 @@ bool TableState::handlePinyinMode(KeyEvent &event) {
                 reset();
                 return true;
             }
-        } else if (event.key().check(FcitxKey_space)) {
+        } else if (event.key().checkKeyList(*config.defaultCandidate)) {
             event.filterAndAccept();
             if (autoSelectCandidate()) {
                 return true;
@@ -866,12 +866,42 @@ void TableState::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
         return;
     }
 
+    if ((mode_ == TableMode::Normal || mode_ == TableMode::Pinyin) &&
+        !event.key().hasModifier()) {
+        auto compose = engine_->instance()->processComposeString(
+            inputContext, event.key().sym());
+        if (!compose) {
+            // invalid compose or in the middle of compose.
+            event.filterAndAccept();
+            return;
+        }
+        // Handle the key just like punc select.
+        if (!compose->empty()) {
+            // if current key will produce some string, select the candidate.
+            if (!autoSelectCandidate()) {
+                commitBuffer(true);
+            }
+            inputContext->commitString(*compose);
+            reset();
+            event.filterAndAccept();
+            return;
+        }
+    }
+
     lastIsPunc_ = false;
 
-    if (handleCandidateList(config, event, needUpdate)) {
+    if (handleCandidateList(config, event)) {
         return;
     }
 
+    // Non candidate key for predict candidate should clear it.
+    if (inputContext->inputPanel().candidateList() &&
+        dynamic_cast<const TablePredictCandidateWord *>(
+            &inputContext->inputPanel().candidateList()->candidate(0))) {
+        inputContext->inputPanel().setCandidateList(nullptr);
+        needUpdate = true;
+    }
+
     // We have special handling of escape here.
     if (handleForgetWord(event)) {
         return;
@@ -937,7 +967,7 @@ void TableState::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
                 CommitAfterSelectWrapper commitAfterSelectRAII(this);
                 context->type(str);
             }
-            if (context->candidates().empty() &&
+            if (!context->dict().hasMatchingWords(context->currentCode()) &&
                 ((*config.commitAfterSelect && context->currentCode() == str) ||
                  (!*config.commitAfterSelect && context->userInput() == str))) {
                 // This means it is not a valid start, make it go through the
@@ -1021,12 +1051,14 @@ void TableState::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
                 needUpdate = true;
             } else if (!context->selected()) {
                 // key to handle when it is not empty.
-                if (event.key().check(FcitxKey_space)) {
-                    if (!autoSelectCandidate()) {
-                        commitBuffer(true);
-                        needUpdate = true;
-                    }
+                if (event.key().checkKeyList(*config.defaultCandidate)) {
                     event.filterAndAccept();
+                    if (autoSelectCandidate()) {
+                        // Update UI is called in candidate::select
+                        return;
+                    }
+                    commitBuffer(true);
+                    needUpdate = true;
                 }
             }
         } else if (event.key().check(FcitxKey_BackSpace) && lastIsPunc) {
@@ -1093,11 +1125,11 @@ void TableState::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
                 engine_->punctuation()
                     ->call<IPunctuation::getPunctuationCandidates>(
                         entry.languageCode(), chr);
+            auto pushResult =
+                engine_->punctuation()->call<IPunctuation::pushPunctuationV2>(
+                    entry.languageCode(), inputContext, chr);
             if (candidates.size() == 1) {
-                std::tie(punc, puncAfter) =
-                    engine_->punctuation()
-                        ->call<IPunctuation::pushPunctuationV2>(
-                            entry.languageCode(), inputContext, chr);
+                std::tie(punc, puncAfter) = pushResult;
             } else if (candidates.size() > 1) {
                 updatePuncCandidate(inputContext, utf8::UCS4ToUTF8(chr),
                                     candidates);
@@ -1123,11 +1155,22 @@ void TableState::keyEvent(const InputMethodEntry &entry, KeyEvent &event) {
 
         if (!punc.empty()) {
             event.filterAndAccept();
-            inputContext->commitString(punc + puncAfter);
-            if (size_t length = utf8::lengthValidated(puncAfter);
-                length != 0 && length != utf8::INVALID_LENGTH) {
-                for (size_t i = 0; i < length; i++) {
-                    inputContext->forwardKey(Key(FcitxKey_Left));
+            auto paired = punc + puncAfter;
+            if (inputContext->capabilityFlags().test(
+                    CapabilityFlag::CommitStringWithCursor)) {
+                if (size_t length = utf8::lengthValidated(punc);
+                    length != 0 && length != utf8::INVALID_LENGTH) {
+                    inputContext->commitStringWithCursor(paired, length);
+                } else {
+                    inputContext->commitString(paired);
+                }
+            } else {
+                inputContext->commitString(paired);
+                if (size_t length = utf8::lengthValidated(puncAfter);
+                    length != 0 && length != utf8::INVALID_LENGTH) {
+                    for (size_t i = 0; i < length; i++) {
+                        inputContext->forwardKey(Key(FcitxKey_Left));
+                    }
                 }
             }
         }
@@ -1224,31 +1267,34 @@ bool TableState::handlePuncCandidate(const TableConfig &config,
         reset();
         return true;
     }
-    int idx = event.key().keyListIndex(config.selection.value());
-    if (idx >= 0) {
-        event.filterAndAccept();
-        if (idx < candidateList->size()) {
-            candidateList->candidate(idx).select(inputContext);
-        }
-        return true;
-    }
 
-    if (auto *movable = candidateList->toCursorMovable()) {
-        if (event.key().checkKeyList(*config.nextCandidate)) {
-            movable->nextCandidate();
-            updatePuncPreedit(inputContext);
-            inputContext->updateUserInterface(
-                UserInterfaceComponent::InputPanel);
+    if (!event.isVirtual()) {
+        int idx = event.key().keyListIndex(config.selection.value());
+        if (idx >= 0) {
             event.filterAndAccept();
+            if (idx < candidateList->size()) {
+                candidateList->candidate(idx).select(inputContext);
+            }
             return true;
         }
-        if (event.key().checkKeyList(*config.prevCandidate)) {
-            movable->prevCandidate();
-            updatePuncPreedit(inputContext);
-            inputContext->updateUserInterface(
-                UserInterfaceComponent::InputPanel);
-            event.filterAndAccept();
-            return true;
+
+        if (auto *movable = candidateList->toCursorMovable()) {
+            if (event.key().checkKeyList(*config.nextCandidate)) {
+                movable->nextCandidate();
+                updatePuncPreedit(inputContext);
+                inputContext->updateUserInterface(
+                    UserInterfaceComponent::InputPanel);
+                event.filterAndAccept();
+                return true;
+            }
+            if (event.key().checkKeyList(*config.prevCandidate)) {
+                movable->prevCandidate();
+                updatePuncPreedit(inputContext);
+                inputContext->updateUserInterface(
+                    UserInterfaceComponent::InputPanel);
+                event.filterAndAccept();
+                return true;
+            }
         }
     }
 
